classdef AutosarUtils
% autosar specific checks
    
%   Copyright 2013-2015 The MathWorks, Inc.

    methods(Static, Access=public)
        
       function msg = isValidIdentifier(cellstrToTest, idType, maxShortNameLength)
       % Function: isValidIdentifier =========================================
       % Abstract:
       %   Validate cell array of AUTOSAR identifier strings

           msg = '';

           for idx = 1:length(cellstrToTest)
               str = cellstrToTest{idx};

               [isvalid, msg] = autosarcore.checkIdentifier(str, idType, maxShortNameLength);
               if ~isvalid || ~isempty(msg)
                   return;
               end

           end

       end

       function checkShortNameCaseClash( shortNames )
       % Function: checkShortNameCaseClash =========================================
       % Abstract: Check for case insensitive clashes among shortNames
       %           (e.g. "RPORT1" and "rport1" clash)

           if isempty(shortNames)
               return
           end
           
           shortNameMap = containers.Map( lower(shortNames), shortNames );
           
           for i=1:length(shortNames)
               shortName1 = shortNames{i};
               shortName2 = shortNameMap( lower(shortName1 ) );
               if ~strcmp( shortName1, shortName2 )
                   DAStudio.error('RTW:autosar:shortNameCaseClash', shortName1, shortName2);
               end
           end
           
       end

       function isNotCKeyword = isNotCKeyword( name )
       % Function: isNotCKeyword ===============================================
       % Abstract:
       %   check NAME is not a c keyword

           dummyConfigEntry = RTW.FcnArgSpec;
           dummyConfigEntry.ArgName = name;

           isNotCKeyword = dummyConfigEntry.isValidIdentifier();
       end
              
       function isErrArg = isErrorArgument(mdlName, Identifier)
       % Function: isErrorArgument ===============================================
       % Abstract:
       %   check NAME is error argument for a function caller or simulink
       %   function.
           isErrArg = false;
           [~, opName, intfQName] = autosar.validation.AutosarUtils.getInterface(mdlName, Identifier);
           if ~isempty(intfQName)
               argName = get_param(Identifier, 'ArgumentName');
               dataobj = autosar.api.getAUTOSARProperties(mdlName, true); 
               direction = dataobj.get([intfQName '/' opName '/' argName], 'Direction');
               isErrArg = strcmp(direction, 'Error');
           end
       end
       
       function [intfName, opName, intfQName] = getInterface(mdlName, Identifier)
       % Function: getInterface ===============================================
       % Abstract:
       %   Determines autosar interface and operation name corresponding to a Simulink
       %   function arg
      
           intfName = '';
           opName = '';
           intfQName = '';
           [parentBlk,~,~] = fileparts(Identifier);
           [~,~,fcnName]  = autosar.validation.ClientServerValidator.getBlockInOutParams(...
               parentBlk);
           mapping = autosar.api.getSimulinkMapping(mdlName);
           runName = mapping.getFunction(fcnName);
           
           dataobj = autosar.api.getAUTOSARProperties(mdlName, true);  
           swc = dataobj.get('XmlOptions', 'ComponentQualifiedName');
           behavior = dataobj.get(swc, 'Behavior', 'PathType', 'FullyQualified');
           runnablePath = [behavior '/' runName];
           opEvents = dataobj.find(behavior, 'OperationInvokedEvent', 'StartOnEvent', runnablePath, 'PathType', 'FullyQualified');
           if length(opEvents) ~= 1
               return;
           end
           trigger = dataobj.get(opEvents{1},'Trigger');
           trigger = strsplit(trigger,'.');
           
           %get server port for operation invoked event
           serverPorts = dataobj.find(swc,'ServerPort','PathType','FullyQualified');
           serverPath = serverPorts(cellfun(@(x)~isempty(x),regexp(serverPorts,['/' trigger{1} '$'])));
           
           %get server port interface
           intfQName = dataobj.get(serverPath{1},'Interface', 'PathType', 'FullyQualified');
           [~, intfName, ~] = fileparts(intfQName);
           opName = trigger{2};
       end
       
       function [propertyValue, interfaceName] = getInterfaceElementProperty(mdlName, portName, elementName, propertyName)
           % Function: getInterfaceElementProperty =============================
           % Abstract:
           %   Returns the value of a port element property.
           
           dataObj = autosar.api.getAUTOSARProperties(mdlName, true);
           swc = dataObj.get('XmlOptions', 'ComponentQualifiedName');
           port = dataObj.find(swc, 'ProvidedPort', 'Name', portName, 'PathType', 'FullyQualified');
           if isempty(port)
               % If a sender port was not provided, try sender-receiver port.
               port = dataObj.find(swc, 'ProvidedRequiredPort', 'Name', portName, 'PathType', 'FullyQualified');
           end
           if iscell(port)
               assert(length(port) == 1);
               port = port{1};
           end
           interfaceName = dataObj.get(port, 'Interface','PathType', 'FullyQualified');
           dataElem = dataObj.find(interfaceName, 'FlowData', 'Name', elementName, 'PathType', 'FullyQualified');
           if iscell(dataElem)
               assert(length(dataElem) == 1);
               dataElem = dataElem{1};
           end
           propertyValue = dataObj.get(dataElem, propertyName);
       end       
       
       function checkDataTypeForErrArg(mdlName, blkPath, ~, ~)
        % Function: checkDataTypeForErrArg ====================================
        % Abstract:
        %   Check header file, name and various settings of error argument

            portHandles = get_param(blkPath, 'PortHandles');
            errPortH = portHandles.Inport(1);

            [~, ~, intfQName] = autosar.validation.AutosarUtils.getInterface(mdlName, blkPath);
            m3iModel = autosar.api.Utils.m3iModel(mdlName);
            arRoot = m3iModel.RootPackage.front();
            intfSeq = autosar.mm.Model.findObjectByNameAndMetaClass(arRoot,...
                intfQName, ...
                Simulink.metamodel.arplatform.interface.ClientServerInterface.MetaClass());
            assert(intfSeq.size() == 1, 'Did not find client server interface');
            m3iInterface = intfSeq.at(1);
            
            autosar.validation.AutosarUtils.checkDataTypeForErrArgPort(errPortH, m3iInterface);
            
        end
            
       function checkDataTypeForErrArgPort(portH, m3iInterface)
        % Function: checkDataTypeForErrArg ====================================
        % Abstract:
        %   Check header file, name and various settings of error argument

            blkPath = get_param(portH, 'Parent');
            
            width = get_param(portH, 'CompiledPortWidth');
            
            if width > 1
               DAStudio.error('autosarstandard:validation:errorArgInvalidWidth',...
                   blkPath);
            end

            dataType = get_param(portH, 'CompiledPortAliasedThruDataType');
            
            aliasDataType = get_param(portH, 'CompiledPortDataType');                       

            if strcmp(dataType, 'uint8')
                % Okay
                return
            elseif Simulink.data.isSupportedEnumClass(dataType)
                if slfeature('ControlStorageTypeForEnums')
                   storageType = Simulink.data.getEnumTypeInfo(dataType, 'StorageType');
                   if ~strcmp( storageType, 'uint8' )
                       DAStudio.error('autosarstandard:validation:errorArgInvalidDataType', blkPath, aliasDataType);
                   end
                end
            else
                DAStudio.error('autosarstandard:validation:errorArgInvalidDataType', blkPath, aliasDataType);
            end

            % Enumeration checks            
            
            % Valid enum check
            errCodes = enumeration(dataType);
            if isempty(errCodes)
                msg = DAStudio.message('RTW:autosar:invalidErrorEnum', dataType);
                DAStudio.error('RTW:fcnClass:finish', msg);
            end
            
            % Range check            
            for ii=1:length(errCodes)
                errCodeValue = uint8(errCodes(ii));                
                if errCodeValue < 0 || errCodeValue > 63
                    msg = DAStudio.message('autosarstandard:validation:invalidAppErrValue', ...
                                           dataType, ...
                        num2str(errCodes(ii)));
                    DAStudio.error('RTW:fcnClass:finish', msg);
                end
            end
            
        end

        
        function checkDataType(mdlName, Identifier, dataTypeName, maxShortNameLength, supportMatrixIOAsArray)
        % Function: checkDataType ====================================
        % Abstract:
        %   Check header file and name settings of Simulink types

        % strip off leading ? or Enum: for enum types
            dataTypeName = regexprep( dataTypeName, '^\?', '');
            dataTypeName = regexprep( dataTypeName, '^Enum:\s*', '');
            Identifier = strtrim(Identifier);
            
            % strip off leading Bus: for Bus types
            dataTypeName = regexprep( dataTypeName, '^Bus:\s*', '');

            % Data name check
            if length(dataTypeName) > maxShortNameLength
                msg = DAStudio.message('RTW:autosar:invalidDataTypeName', Identifier, dataTypeName, maxShortNameLength, maxShortNameLength);
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            if existsInGlobalScope(mdlName, dataTypeName)
                slObj = globalScopeDataItem(mdlName, dataTypeName);

                % Check for header file at root level
                if isa(slObj, 'Simulink.AliasType') || ...
                   isa(slObj, 'Simulink.NumericType') || ...
                   isa(slObj, 'Simulink.StructType')
                    if ~strcmp( slObj.HeaderFile, 'Rte_Type.h' )
                        msg = DAStudio.message('RTW:autosar:invalidDataTypeHeader', Identifier, dataTypeName);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                    if ~isempty(strmatch('DataScope', fieldnames(slObj))) && ...
                            strcmp( slObj.DataScope, 'Exported' ) %#ok<MATCH2>
                        msg = DAStudio.message('RTW:autosar:invalidDataTypeScope', Identifier, dataTypeName);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                else
                    % okay to fall through
                end

                % now attempt to recurse through datatype elements/base types etc.
                if isa(slObj, 'Simulink.AliasType')
                    % check base type
                    if strcmp(dataTypeName, slObj.BaseType)
                        % Skip check for self referencing aliastypes
                    else
                        autosar.validation.AutosarUtils.checkDataType(mdlName, Identifier, slObj.BaseType, maxShortNameLength, supportMatrixIOAsArray);
                    end
                elseif isa(slObj, 'Simulink.NumericType')
                    if slObj.isfixed && slObj.WordLength > 32
                        DAStudio.error('RTW:autosar:incorrectExportedWordSize', ...
                                       dataTypeName, ...
                                       int2str(slObj.WordLength));
                    end
                elseif isa(slObj, 'Simulink.StructType')

                    % make sure StructType Elements are arrays rather than matrices
                    if ~supportMatrixIOAsArray
                        autosar.validation.AutosarUtils.checkStructTypeDimension(mdlName, slObj, Identifier, dataTypeName);
                    end

                    % recurse through elements
                    for elemIdx=1:length(slObj.Elements)
                        elem = slObj.Elements(elemIdx);

                        % check each element
                        autosar.validation.AutosarUtils.checkDataType(mdlName, Identifier, elem.DataType, maxShortNameLength, supportMatrixIOAsArray);
                    end
                else
                    % okay to fall through
                end
            end

            if isvarname(dataTypeName)
                if Simulink.data.isSupportedEnumClass(dataTypeName)
                    headerFile = Simulink.data.getEnumTypeInfo(dataTypeName, 'HeaderFile');
                    if ~strcmp( headerFile, 'Rte_Type.h' )
                        msg = DAStudio.message('RTW:autosar:invalidEnumHeader', dataTypeName);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                    dataScope = Simulink.data.getEnumTypeInfo(dataTypeName, 'DataScope');
                    if strcmp( dataScope, 'Exported' )
                        msg = DAStudio.message('RTW:autosar:invalidEnumScope', dataTypeName);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                    cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(mdlName);
                    if ~AR4.isCompatibleSchemaVer(get_param(cs, 'AutosarSchemaVersion'))
                        storageType = Simulink.data.getEnumTypeInfo(dataTypeName, 'StorageType');
                        if ~strcmp(storageType, 'int')
                            DAStudio.error('autosarstandard:validation:enumWithStorageTypeNotAllowed',...
                                Identifier, dataTypeName, storageType);
                        end
                    end
                else
                    if strncmp(dataTypeName, 'ufix', 4) || strncmp(dataTypeName, 'sfix', 4)
                        % extract WordLength
                        output = regexp(dataTypeName, 'fix(?<WordLength>\d+)', 'names');
                        if str2double(output.WordLength) > 32
                            DAStudio.error('RTW:autosar:incorrectExportedWordSize', ...
                                           dataTypeName, ...
                                           num2str(output.WordLength));
                        end                
                    end
                end
            end

        end

        function str = removeNewLine( str )
        % Function: removeNewLine =================================================
        % Abstract:
            str = strrep(str, sprintf('\n'), ' ');
        end       
        
        function [isValid, errmsg] = checkSymbol(str)
        % Function: checkSymbol =================================================
        % Abstract: Check if the input string is a valid AUTOSAR symbol
            
            % Check if symbol is conflict with C keywords
            % and check if symbol is a valid C identifier
            isValid = ...
                autosar.validation.AutosarUtils.isNotCKeyword(str) && ...
                all(regexp(str, '[_a-zA-Z][_a-zA-Z0-9]*'));
            % Generate error message
            if isValid == true
                errmsg = [];
            else
                errmsg = DAStudio.message('RTW:autosar:invalidSymbol', str);
            end
        end
        
        function  [isValid, errmsg] = checkDisplayFormat(str, identifier)
            isValid = false;
            errmsg = [];
            if ~isempty(str)
                % % [flags] [width] [.prec] type character
                expression = '%[ \-+#]?[0-9]*(\.[0-9])?[diouxXfeEgGcs]';
                [startRegex, endRegex] = regexp(str, expression, 'ONCE');
                if ~isempty(startRegex) && startRegex == 1 && numel(str) == endRegex
                    isValid = true;
                else
                    errmsg = DAStudio.message('autosarstandard:ui:validateDisplayFormat', identifier);
                end
            else
              isValid = true;
            end
        end
        
    end        
    
    methods(Static, Access=private)
        
        function checkStructTypeDimension(mdlName, slObj, Identifier, dataTypeName)
        % Function: checkStructTypeDimension ====================================
        % Abstract:
        %   Check dimension of struct type is n, [1xn] or [nx1]

            for elemIdx=1:length(slObj.Elements)
                elem = slObj.Elements(elemIdx);
                if length(elem.Dimensions) > 1
                    if (length(elem.Dimensions) == 2 && (elem.Dimensions(1) == 1 || ...
                                                         elem.Dimensions(2) == 1))
                        % okay
                    else
                        msg = DAStudio.message('RTW:autosar:invalidStructTypeDimension', Identifier, dataTypeName);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                end

                % recurse through elements
                if existsInGlobalScope(mdlName, elem.DataType)
                    elemObj = evalinGlobalScope(mdlName, elem.DataType);
                    if isa(elemObj, 'Simulink.StructType')
                        autosar.validation.AutosarUtils.checkStructTypeDimension(mdlName, elemObj, Identifier, elem.DataType);
                    else
                        % do nothing for everything else
                    end
                end
            end
        end

    end
    
end

% LocalWords:  CKeyword xn nx RPORT rport aliastypes autosarstandard prec dioux
% LocalWords:  Xfe Eg Compu Thru
