classdef CompiledModelUtils
% utility functions for dealing with a compiled Simulink model
    
%   Copyright 2013 The MathWorks, Inc.

    methods(Static, Access=public)

        function cleanupFuncObj = forceCompiledModel(hModel)
        
            needTerm = false;
            
            fullname = getfullname(hModel);
            if ~autosar.validation.CompiledModelUtils.isCompiled(hModel)
                try
                    feval(fullname, [], [], [], 'compile');
                    needTerm = true;
                catch me
                    if autosar.validation.ExportFcnValidator.isVirtualSubsysExportFcn(hModel)
                        DAStudio.error('RTW:autosar:modelNotCompile', me.message, fullname);
                    else
                        rethrow(me);
                    end
                end
            end
            
            cleanupFuncObj = [];
            if needTerm
                cleanupFuncObj = onCleanup(@()feval(getfullname(hModel), [], [], [], 'term'));
            end
            
            
        end
        
        function cleanupFuncObj = forceCompiledModelForRTW(hModel)
        % Call this function when validating
            needTerm = false;
            
            fullname = getfullname(hModel);
            if ~autosar.validation.CompiledModelUtils.isCompiled(hModel)
                try
                    feval(fullname, [], [], [], 'compileForRTW');
                    needTerm = true;
                catch me
                    if autosar.validation.ExportFcnValidator.isVirtualSubsysExportFcn(hModel)
                        DAStudio.error('RTW:autosar:modelNotCompile', me.message, fullname);
                    else
                        rethrow(me);
                    end
                end
            end
            
            cleanupFuncObj = [];
            if needTerm
                cleanupFuncObj = onCleanup(@()feval(getfullname(hModel), [], [], [], 'term'));
            end
            
            
        end
        
        function isCompiled = isCompiled(hModel)
        
            simStatus = get_param(hModel, 'SimulationStatus');
            isCompiled = strcmpi(simStatus, 'paused') || ...
                strcmpi(simStatus, 'initializing') || ...
                strcmpi(simStatus,'running') || ...
                strcmpi(simStatus,'updating');
        
        end
        
        function wsVar = getReferencedWSVars(hModel)

            assert(~strcmp(get_param(hModel,'SimulationStatus'),'stopped'), ...
                   'The model must be compiled');

            mode = 'cached'; % Cached mode is faster but the model must be compiled

            % get list of referenced workspace variables    
            vars = Simulink.findVars(get_param(hModel, 'Name'), 'SearchMethod', mode);
            wsVar = [];
            for i=1:length(vars)
                if strcmpi(vars(i).SourceType, 'data dictionary') || ...
                        strcmpi(vars(i).SourceType, 'base workspace')
                    wsVar(end+1).objName = vars(i).Name; %#ok
                    wsVar(end).obj = evalinGlobalScope(hModel, vars(i).Name);
                end
            end
        end 
        
        
        function portInfo = getPortInfo(hSrc)
        % Cell array for storing {PortDataTypes, PortWidths, isInport, PortSampleTimes, PortName, AliasedPortDataTypes}
            
            configData = hSrc.syncWithModel();
            hModel = hSrc.ModelHandle;
            
            inpH = find_system(hModel, 'SearchDepth', 1, 'Type', 'block',...
                               'BlockType', 'Inport');
            outpH = find_system(hModel, 'SearchDepth', 1, 'Type', 'block',...
                                'BlockType', 'Outport');

            
            % Cell array for storing {PortDataTypes, PortWidths, isInport, PortSampleTimes, PortName, AliasedPortDataTypes}
            portInfo = cell(length(configData), 6);

            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            supportMatrixIOAsArray = strcmpi(...
                get_param(cs,'AutosarMatrixIOAsArray'), 'on');

            for i=1:length(configData)
                if isscalar(configData)
                    entry = configData;
                else
                    entry = configData(i);
                end
                if strcmp(entry.SLObjectType, 'Inport')
                    isInport = true;
                    portH = inpH(entry.PortNum); %PortNum is 1 based
                    portHDims = get_param(portH, 'CompiledPortDimensions');
                    if ~supportMatrixIOAsArray && ...
                            (portHDims.Outport(1) > 1) && ...
                            (sum(portHDims.Outport(2:end) ~= 1) > 1)
                        msg = DAStudio.message('RTW:autosar:scalarOnly', ...
                                               get_param(portH, 'Name'));
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end

                    % Get the DataType and Width
                    portHDataTypes = get_param(portH, 'CompiledPortAliasedThruDataTypes');
                    portHDataTypesAliased = get_param(portH, 'CompiledPortDataTypes');
                    portHWidths = get_param(portH, 'CompiledPortWidths');
                    portInfo{i,1} = portHDataTypes.Outport{1};
                    portInfo{i,2} = portHWidths.Outport(1);
                    portInfo{i,3} = isInport;
                    portInfo{i,4} = get_param(portH, 'CompiledSampleTime');
                    portInfo{i,5} = arblk.convertPortNameToArgName(get_param(portH, 'Name'));
                    portInfo{i,6} = portHDataTypesAliased.Outport{1};
                    portInfo{i,7} = portH;
                    
                else %Outport
                    isInport = false;
                    portH = outpH(entry.PortNum); %PortNum is 1 based
                    portHDims = get_param(portH, 'CompiledPortDimensions');
                    if ~supportMatrixIOAsArray && ...
                            (portHDims.Inport(1) > 1) && ...
                            (sum(portHDims.Inport(2:end) ~= 1) > 1)
                        msg = DAStudio.message('RTW:autosar:scalarOnly', get_param(portH, 'Name'));
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end

                    % Get the DataType and Width
                    portHDataTypes = get_param(portH, 'CompiledPortAliasedThruDataTypes');
                    portHDataTypesAliased = get_param(portH, 'CompiledPortDataTypes');
                    portHWidths = get_param(portH, 'CompiledPortWidths');
                    portInfo{i,1} = portHDataTypes.Inport{1};
                    portInfo{i,2} = portHWidths.Inport(1);
                    portInfo{i,3} = isInport;
                    portInfo{i,4} = get_param(portH, 'CompiledSampleTime');
                    portInfo{i,5} = arblk.convertPortNameToArgName(get_param(portH, 'Name'));
                    portInfo{i,6} = portHDataTypesAliased.Inport{1};
                    portInfo{i,7} = portH;

                end
            end
            
        end
        

    end        

    
end

% LocalWords:  Thru
