classdef DataObjectValidator < autosar.validation.PhasedValidator
% Reusable validation functions for AUTOSAR

%   Copyright 2013-2015 The MathWorks, Inc.

    properties(Access=private)
        CalibPortMap;      % hash table for calibration port checks
    end    

    methods (Access=protected)
        
        function verifyPostProp(this, hModel)
            this.CalibPortMap = containers.Map;
            
            % Require compiled model                            
            this.verifyCalPrms(hModel);
            this.verifyInternalCalPrms(hModel);
            this.verifyConstantMemoryPrms(hModel);
            this.verifyStaticMemorySignals(hModel);
            this.verifyDSMPIMs(hModel);
        end     
    end
    
    methods (Static, Access=public)
        
        function dsmPIMNames = getDSMPIMNames(hModel)
            [~, dsmPIMNames] = autosar.validation.DataObjectValidator.getReferencedDSMPIMs(hModel);
        end
        
        function internalCalPrmNames = getInternalCalPrmNames(hModel)
            [~, internalCalPrmNames] = autosar.validation.DataObjectValidator.getReferencedInternalCalPrms(hModel);
        end
        
        function constMemPrmNames = getConstMemoryPrmNames(hModel)
            [~, constMemPrmNames] = autosar.validation.DataObjectValidator.getReferencedConstantMemoryParams(hModel);
        end
        
        function staticMemoryVarNames = getStaticMemoryVarNames(hModel)
            [~, staticMemoryVarNames] = autosar.validation.DataObjectValidator.getReferencedStaticMemoryVars(hModel);
        end        

    end
    
    methods (Access=private)
        function verifyCalPrms(this, hModel)
            % Function: verifyCalPrmObjs =========================================
            % Abstract: [Compiled] Verify CalPrm objects for AUTOSAR
            
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            isAR4 = AR4.isCompatibleSchemaVer(get_param(cs, 'AutosarSchemaVersion'));
            maxShortNameLength = get_param(cs, 'AutosarMaxShortNameLength');  
            supportMatrixIOAsArray = strcmpi(...
                get_param(cs,'AutosarMatrixIOAsArray'), 'on');
            
            % get list of referenced calibration parameters
            vars = autosar.validation.CompiledModelUtils.getReferencedWSVars(hModel);
            calibComponents = cell(length(vars), 0);
            calibInterfaces = cell(length(vars), 0);
            calibInterfacePkgs = cell(length(vars), 0);
            calibComponentPkgs = cell(length(vars), 0);

            nonParamRPortNames = {};
            if autosar.api.Utils.isMapped(hModel)
                m3iModel = autosar.api.Utils.m3iModel(hModel);
                arRoot = m3iModel.RootPackage.front();
                m3iComp = autosar.mm.Model.findChildByName(arRoot, ...
                                                       arRoot.ComponentQualifiedName);
                    
                % Cache non parameter receiver port names
                nonParamRPorts = m3i.filter(@(obj) ~isa(obj, 'Simulink.metamodel.arplatform.port.ParameterReceiverPort'), m3iComp.Port);
                nonParamRPortNames = cellfun(@(obj) obj.Name, nonParamRPorts, 'UniformOutput', false);
            end
            
            for i=1:length(vars)
                obj     = vars(i).obj;
                objName = vars(i).objName;
                
                % Check if value is an AUTOSAR CalPrm
                if ( ...
                    (isa( obj, 'AUTOSAR.Parameter' ) && ...
                     strcmp( vars(i).obj.CoderInfo.StorageClass, 'Custom' ) && ...
                     strcmp( obj.CoderInfo.CustomStorageClass, 'CalPrm' )) ...
                    || ...
                    (isa(vars(i).obj,'Simulink.Parameter') && ...
                     strcmp( vars(i).obj.CoderInfo.StorageClass, 'Custom' ) && ...
                     isa(vars(i).obj.CoderInfo.CustomAttributes,'SimulinkCSC.AttribClass_AUTOSAR_CalPrm')) ...
                    )
                    attr = obj.CoderInfo.CustomAttributes;
                    if strcmpi(vars(i).obj.DataType, 'struct')
                        DAStudio.warning('RTW:autosar:calPrmHasAnonymousStructType', vars(i).objName);
                    end

                    autosar.validation.AutosarUtils.checkDataType(hModel, ['CalPrm ' objName], vars(i).obj.DataType, maxShortNameLength, supportMatrixIOAsArray);

                    % RuleSNId9
                    idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier( {attr.PortName}, 'shortName', maxShortNameLength);
                    if ~isempty(idcheckmessage)
                        msg = DAStudio.message('RTW:autosar:externalCalPrm', objName, 'PortName', idcheckmessage);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end

                    % RuleSNId10
                    idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier( {attr.ElementName}, 'shortName', maxShortNameLength);
                    if ~isempty(idcheckmessage)
                        msg = DAStudio.message('RTW:autosar:externalCalPrm', objName, 'ElementName', idcheckmessage);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end

                    % RulePRId3
                    idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier( {attr.InterfacePath}, 'absPathShortName', maxShortNameLength);
                    if ~isempty(idcheckmessage)
                        msg = DAStudio.message('RTW:autosar:externalCalPrm', objName, 'InterfacePath', idcheckmessage);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                    if ~strcmp(attr.InterfacePath, 'UNDEFINED')
                        calibInterfaces{i} = attr.InterfacePath;
                        [package,~]=autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(attr.InterfacePath);
                        calibInterfacePkgs{i} = package;    
                    end
                    if ~isempty(attr.CalibrationComponent)
                        calibComponents{i} = attr.CalibrationComponent;
                        [package,~] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(attr.CalibrationComponent);
                        calibComponentPkgs{i} = package;
                    end                    
                    % Check CalibrationComponent and ProviderPortName
                    % RulePRId4
                    if ~isempty(attr.CalibrationComponent) || ~isempty(attr.ProviderPortName)
                        idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier(...
                            {attr.CalibrationComponent}, 'absPathShortName', maxShortNameLength);
                        if ~isempty(idcheckmessage)
                            msg = DAStudio.message('RTW:autosar:externalCalPrm', objName,...
                                'CalibrationComponent', idcheckmessage);
                            DAStudio.error('RTW:fcnClass:finish', msg);
                        end
                        
                        idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier(...
                            {attr.ProviderPortName}, 'shortName', maxShortNameLength);
                        if ~isempty(idcheckmessage)
                            msg = DAStudio.message('RTW:autosar:externalCalPrm', objName,...
                                'ProviderPortName', idcheckmessage);
                            DAStudio.error('RTW:fcnClass:finish', msg);
                        end
                        
                        % RulePRId5
                        % If the meta-model already has an object with the
                        % same path as the specified CalibrationComponent,
                        % then this object should must be of type
                        % "ParameterComponent"
                        if autosar.api.Utils.isMapped(hModel)
                            m3iModel = autosar.api.Utils.m3iModel(hModel);
                            compObj = autosar.mm.Model.findObjectByName(m3iModel, attr.CalibrationComponent);
                            if ~isempty(compObj) && compObj.size > 0
                                % check that it is of type ParameterComponent
                                compObj = compObj.at(1);
                                if ~isa(compObj, 'Simulink.metamodel.arplatform.component.ParameterComponent')
                                    msg = DAStudio.message('RTW:autosar:CalPrmProviderComponentConflict',...
                                        attr.CalibrationComponent,objName, attr.CalibrationComponent);
                                    DAStudio.error('RTW:fcnClass:finish', msg);
                                end
                            end
                        end
                    end
                    
                    % Check RPort conflicts
                    this.checkCalibPortConflicts('RPort', attr, objName, nonParamRPortNames);
                    
                    % Check PPort conflicts
                    if ~isempty(attr.ProviderPortName)
                        this.checkCalibPortConflicts('PPort', attr, objName, {});
                    end
                    if isAR4
                        autosar.validation.DataObjectValidator.verifyDisplayFormat(['CalPrm ' objName], vars(i).obj);                    
                    end
                end
            end             
            this.checkPackageClash('Calibration interface', calibInterfacePkgs, 'calibration component', calibComponents, vars);
            this.checkPackageClash('Calibration interface', calibInterfacePkgs, 'calibration interface', calibInterfaces, vars);
            this.checkPackageClash('Calibration component', calibComponentPkgs, 'calibration interface', calibInterfaces, vars);
            this.checkPackageClash('Calibration component', calibComponentPkgs, 'calibration component', calibComponents, vars);
            if autosar.api.Utils.isMapped(hModel)
                m3iModel = autosar.api.Utils.m3iModel(hModel);
                for ii = 1 : numel(calibInterfaces)
                    if ~isempty(calibInterfaces{ii})
                        calibInterface = autosar.mm.Model.findObjectByName(m3iModel, calibInterfaces{ii});
                        if calibInterface.size() > 0
                            if ~isa(calibInterface.at(1), 'Simulink.metamodel.arplatform.interface.ParameterInterface')
                                msg = DAStudio.message('autosarstandard:ui:packageClashCalibParamErr',...
                                    'calibration interface', objName, attr.InterfacePath);
                                DAStudio.error('RTW:fcnClass:finish', msg);
                            end
                        end
                    end
                end
            end
        end

        function checkPackageClash(this, pkgType, pkgs, qType, qNames, vars) %#ok<INUSL>
            [indexSet1, indexSet2] = autosar.mm.util.findAmbigousPkgElementNames(...
                pkgs, qNames);
            if ~isempty(indexSet1) && ~isempty(indexSet2)
                if indexSet1{1} == indexSet2{1}
                    msg = DAStudio.message('autosarstandard:ui:packageClashCalibParamErr',...
                        [qType ' ' qNames{indexSet2{1}}], vars(indexSet2{1}).objName,...
                        pkgType);
                else
                    msg = DAStudio.message('autosarstandard:ui:packageClashWithOtherCalibParamErr',...
                        [qType ' ' qNames{indexSet2{1}}], vars(indexSet2{1}).objName,...
                        pkgType, vars(indexSet1{1}).objName);
                end
                DAStudio.error('RTW:fcnClass:finish', msg);
            end            
        end
        
        function checkCalibPortConflicts(this, portType, attr, objName, excPortNames)
            % Push port onto port map if it doesn't exist, o.w. check 
            % that the existing port has same type and interface path 
            % and issue error if not.
            switch (portType)
                case 'RPort'
                    portName = attr.PortName;                   
                    
                    if any(strcmp(portName, excPortNames))
                        DAStudio.error('RTW:autosar:errorDuplicatePort', portName);
                    end
                    
                    
                case 'PPort'
                    portName = [attr.CalibrationComponent '/' attr.ProviderPortName];
                otherwise
                    assert('Unexpected portType "%s".', portType);
            end
            
            objNameMsg = DAStudio.message('RTW:autosar:workspaceVariable', objName);
            if ~(this.CalibPortMap.isKey(portName))
                % add to port map
                this.CalibPortMap(portName) = struct(...
                    'interfacePath', attr.interfacePath,...
                    'elementName', attr.elementName,...
                    'objNameMsg', objNameMsg);
                
                % case insensitive check
                autosar.validation.AutosarUtils.checkShortNameCaseClash(...
                    this.CalibPortMap.keys);
            else
                % check interfacePaths
                if ~strcmp(attr.interfacePath, this.CalibPortMap(portName).interfacePath )
                    msg = DAStudio.message('RTW:autosar:portHasDuplicateInterfacePath',...
                        portName, objNameMsg, attr.interfacePath,...
                        this.CalibPortMap(portName).interfacePath,...
                        this.CalibPortMap(portName).objNameMsg );
                    DAStudio.error('RTW:fcnClass:finish', msg);
                end
                
                % check elementNames
                if strcmp(attr.elementName, this.CalibPortMap(portName).elementName)
                    DAStudio.error('autosarstandard:validation:calprmDuplicateDataElement',...
                        attr.elementName, portName, objNameMsg, ...
                        this.CalibPortMap(portName).objNameMsg );
                end
                
            end
        end
    end
    methods (Static, Access=private)                        
        function verifyDisplayFormat(identifier, obj)
            if isa(obj, 'AUTOSAR.Signal') || isa(obj, 'AUTOSAR.Parameter')
                if ~isempty(obj.DisplayFormat)
                    [isValid, msg] = autosar.validation.AutosarUtils.checkDisplayFormat(obj.DisplayFormat, identifier);
                    if ~isValid
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                end
            end
        end        
        function verifyDSMPIMs(hModel)
        % Function: verifyDSMPIMs =========================================
        % Abstract: [Compiled] Verify DSM PIM objects for AUTOSAR
            
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            isAR4 = AR4.isCompatibleSchemaVer(get_param(cs, 'AutosarSchemaVersion'));
            maxShortNameLength = get_param(cs, 'AutosarMaxShortNameLength');
            supportMatrixIOAsArray = strcmpi(...
                get_param(cs,'AutosarMatrixIOAsArray'), 'on');
            
            [vars, identifiers] = autosar.validation.DataObjectValidator.getReferencedDSMPIMs(hModel);
            for i=1:length(vars)
                identifier = identifiers{i};
                
                autosar.validation.AutosarUtils.checkDataType(hModel, ['PerInstanceMemory ' identifier], ...
                                                              vars(i).obj.DataType, maxShortNameLength, ...
                                                              supportMatrixIOAsArray);
                    
                idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier( {identifier}, 'shortName', maxShortNameLength);
                if ~isempty(idcheckmessage)
                    msg = DAStudio.message('RTW:autosar:perInstanceMemory', identifier, idcheckmessage);
                    DAStudio.error('RTW:fcnClass:finish', msg);
                end
                if isAR4
                    autosar.validation.DataObjectValidator.verifyDisplayFormat(['PerInstanceMemory ' identifier], vars(i).obj);
                end
            end
        end

        
        function verifyInternalCalPrms(hModel)
        % Function: verifyInternalCalPrms =========================================
        % Abstract: [Compiled] Verify InternalCalPrm objects for AUTOSAR
            
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            isAR4 = AR4.isCompatibleSchemaVer(get_param(cs, 'AutosarSchemaVersion'));
            maxShortNameLength = get_param(cs, 'AutosarMaxShortNameLength');
            supportMatrixIOAsArray = strcmpi(...
                get_param(cs,'AutosarMatrixIOAsArray'), 'on');
            
            [vars, identifiers] = autosar.validation.DataObjectValidator.getReferencedInternalCalPrms(hModel);
            for i=1:length(vars)
                identifier = identifiers{i};

                if strcmpi(vars(i).obj.DataType, 'struct')
                    DAStudio.warning('RTW:autosar:calPrmHasAnonymousStructType', vars(i).objName);
                end
                autosar.validation.AutosarUtils.checkDataType(hModel, ['InternalCalPrm ' identifier], vars(i).obj.DataType, maxShortNameLength, supportMatrixIOAsArray);

                idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier( {identifier}, 'shortName', maxShortNameLength);
                if ~isempty(idcheckmessage)
                    msg = DAStudio.message('RTW:autosar:internalCalPrm', idcheckmessage);
                    DAStudio.error('RTW:fcnClass:finish', msg);
                end
                if isAR4
                    autosar.validation.DataObjectValidator.verifyDisplayFormat(['InternalCalPrm ' identifier], vars(i).obj);
                end
            end
        end

        function verifyConstantMemoryPrms(hModel)
            % Function: verifyConstMemoryPrms =========================================
            % Abstract: [Compiled] Verify Constant Memory Parameter objects for AUTOSAR
            
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            isAR4 = AR4.isCompatibleSchemaVer(get_param(cs, 'AutosarSchemaVersion'));
            if isAR4
                maxShortNameLength = get_param(cs, 'AutosarMaxShortNameLength');
                supportMatrixIOAsArray = strcmpi(...
                    get_param(cs,'AutosarMatrixIOAsArray'), 'on');
                
                [vars, identifiers] = autosar.validation.DataObjectValidator.getReferencedConstantMemoryParams(hModel);
                for i=1:length(vars)
                    identifier = identifiers{i};
                    
                    if strcmpi(vars(i).obj.DataType, 'struct')
                        DAStudio.warning('RTW:autosar:calPrmHasAnonymousStructType', vars(i).objName);
                    end
                    
                    autosar.validation.AutosarUtils.checkDataType(hModel, ['ConstantMemory ' identifier], vars(i).obj.DataType, maxShortNameLength, supportMatrixIOAsArray);
                    
                    idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier( {identifier}, 'shortName', maxShortNameLength);
                    if ~isempty(idcheckmessage)
                        msg = DAStudio.message('RTW:autosar:internalCalPrm', idcheckmessage);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                    autosar.validation.DataObjectValidator.verifyDisplayFormat(['ConstantMemory ' identifier], vars(i).obj)
                end
            end
        end
        
        function verifyStaticMemorySignals(hModel)
            % Function: verifyStaticMemorySignals =========================================
            % Abstract: [Compiled] Verify Static Memory Signal objects for AUTOSAR
            
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            isAR4 = AR4.isCompatibleSchemaVer(get_param(cs, 'AutosarSchemaVersion'));
            if isAR4
                maxShortNameLength = get_param(cs, 'AutosarMaxShortNameLength');
                supportMatrixIOAsArray = strcmpi(...
                    get_param(cs,'AutosarMatrixIOAsArray'), 'on');
                
                [vars, identifiers] = autosar.validation.DataObjectValidator.getReferencedStaticMemoryVars(hModel);
                for i=1:length(vars)
                    identifier = identifiers{i};
                    
                    autosar.validation.AutosarUtils.checkDataType(hModel, ['StaticMemory ' identifier], vars(i).obj.DataType, maxShortNameLength, supportMatrixIOAsArray);
                    
                    idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier( {identifier}, 'shortName', maxShortNameLength);
                    if ~isempty(idcheckmessage)
                        msg = DAStudio.message('RTW:autosar:staticMemory', idcheckmessage);
                        DAStudio.error('RTW:fcnClass:finish', msg);
                    end
                    autosar.validation.DataObjectValidator.verifyDisplayFormat(['StaticMemory ' identifier], vars(i).obj);
                end
            end
        end
        
        function [dsmPimVars, dsmPimVarNames] = getReferencedDSMPIMs(hModel)
        % Function: getReferencedDSMPIMs =========================================
        % Abstract: [Compiled] Get PIM objects
            
        % get list of referenced PIM DSM's and their names
            dsmPimVars = [];
            dsmPimVarNames = {};

            vars = autosar.validation.CompiledModelUtils.getReferencedWSVars(hModel);
            for i=1:length(vars)
                if ( (isa(vars(i).obj,'AUTOSAR.Signal') && ...
                      vars(i).obj.getIsAutosarPerInstanceMemory()) ...
                     || ...
                     ( isa(vars(i).obj,'Simulink.Signal') && strcmp( vars(i).obj.CoderInfo.StorageClass, 'Custom') && isa(vars(i).obj.CoderInfo.CustomAttributes,'SimulinkCSC.AttribClass_AUTOSAR_PerInstanceMemory')) )
                    % both PIM and InternalCalPrm can use alias name
                    if ~isempty(vars(i).obj.CoderInfo.Alias)
                        identifier = vars(i).obj.CoderInfo.Alias;
                    else
                        identifier = vars(i).objName;
                    end
                        
                    if isempty(dsmPimVars)
                        dsmPimVars = vars(i);
                    else                  
                        dsmPimVars(end+1) = vars(i); %#ok<AGROW>
                    end
                    dsmPimVarNames{end+1} = identifier; %#ok<AGROW>
                end

            end
        
        end

        function [internalCalPrmVars, internalCalPrmNames] = getReferencedInternalCalPrms(hModel)
        % Function: getReferencedInternalCalPrms =========================================
        % Abstract: [Compiled] Verify InternalCalPrm objects for AUTOSAR
            
        % get list of referenced PIM DSM's and their names
            internalCalPrmVars = [];
            internalCalPrmNames = {};
            
            vars = autosar.validation.CompiledModelUtils.getReferencedWSVars(hModel);
            for i=1:length(vars)
                if ( ...
                    (isa(vars(i).obj,'AUTOSAR.Parameter') && ...
                     strcmp( vars(i).obj.CoderInfo.StorageClass, 'Custom' ) && ...
                     strcmp( vars(i).obj.CoderInfo.CustomStorageClass, 'InternalCalPrm' )) ...
                    || ...
                    ( isa(vars(i).obj,'Simulink.Parameter') && ...
                      strcmp( vars(i).obj.CoderInfo.StorageClass, 'Custom' ) && ...
                      isa(vars(i).obj.CoderInfo.CustomAttributes,'SimulinkCSC.AttribClass_AUTOSAR_InternalCalPrm')) ...
                    )
                    % both PIM and InternalCalPrm can use alias name
                    if ~isempty(vars(i).obj.CoderInfo.Alias)
                        identifier = vars(i).obj.CoderInfo.Alias;
                    else
                        identifier = vars(i).objName;
                    end

                    if isempty(internalCalPrmVars)
                        internalCalPrmVars = vars(i);
                    else                  
                        internalCalPrmVars(end+1) = vars(i); %#ok<AGROW>
                    end
                    internalCalPrmNames{end+1} = identifier; %#ok<AGROW>
                end
            end
        end
        
        function [constMemPrmVars, constMemNames] = getReferencedConstantMemoryParams(hModel)
            % Function: getReferencedConstantMemoryParams =========================================
            % Abstract: [Compiled] Verify Constant Memory objects for AUTOSAR
            
            % get list of referenced constant memory parameters and their names
            constMemPrmVars = [];
            constMemNames = {};
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            isAR4 = AR4.isCompatibleSchemaVer(get_param(cs, 'AutosarSchemaVersion'));
            if isAR4
                vars = autosar.validation.CompiledModelUtils.getReferencedWSVars(hModel);
                modelName = get_param(hModel, 'Name');
                for i=1:length(vars)
                    if (isa(vars(i).obj,'Simulink.Parameter') && autosar.mm.util.getIsAutosarConstantMemory(modelName, vars(i).objName) )
                        % Constant Memory param can use alias name
                        if ~isempty(vars(i).obj.CoderInfo.Alias)
                            identifier = vars(i).obj.CoderInfo.Alias;
                        else
                            identifier = vars(i).objName;
                        end
                        
                        if isempty(constMemPrmVars)
                            constMemPrmVars = vars(i);
                        else
                            constMemPrmVars(end+1) = vars(i); %#ok<AGROW>
                        end
                        constMemNames{end+1} = identifier; %#ok<AGROW>
                    end
                end
            end
        end

        function [staticMemSignalVars, staticMemNames] = getReferencedStaticMemoryVars(hModel)
            % Function: getReferencedConstantMemoryParams =========================================
            % Abstract: [Compiled] Verify Constant Memory objects for AUTOSAR
            
            % get list of referenced constant memory parameters and their names
            staticMemSignalVars = [];
            staticMemNames = {};
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            isAR4 = AR4.isCompatibleSchemaVer(get_param(cs, 'AutosarSchemaVersion'));
            if isAR4
                modelName = get_param(hModel, 'Name');
                vars = autosar.validation.CompiledModelUtils.getReferencedWSVars(hModel);
                for i=1:length(vars)
                    if (isa(vars(i).obj,'Simulink.Signal') && autosar.mm.util.getIsAutosarStaticMemory(modelName, vars(i).objName) )
                        % Static Memory signal can use alias name
                        if ~isempty(vars(i).obj.CoderInfo.Alias)
                            identifier = vars(i).obj.CoderInfo.Alias;
                        else
                            identifier = vars(i).objName;
                        end
                        
                        if isempty(staticMemSignalVars)
                            staticMemSignalVars = vars(i);
                        else
                            staticMemSignalVars(end+1) = vars(i); %#ok<AGROW>
                        end
                        staticMemNames{end+1} = identifier; %#ok<AGROW>
                    end
                end
            end
        end
        
    end

end

% LocalWords:  Objs Attrib DSMPI Prms DSM's metamodel arplatform RPort PPort
% LocalWords:  autosarstandard Calib calprm
