classdef ServerOperationValidator < autosar.validation.PhasedValidator
% Validation functions for AUTOSAR server operations functionality. AUTOSAR
% server operation validation only needs to be performed with AutosarInterface.

%   Copyright 2013 The MathWorks, Inc.
    
    methods (Access=protected)
        
        function verifyInitial(this, hSrc)
            this.verifyNotRightClickBuild(hSrc);
            this.verifyShortNames(hSrc);
            this.verifyConfigSet(hSrc);        
        end
        
        function verifyPostProp(this, hSrc)
            this.verifyServerOperationPortCheck(hSrc);
            this.verifyNoSimulinkGlobals(hSrc);            
        end

    end

    methods (Static, Access=private)

        function verifyNotRightClickBuild(hSrc)
        % IsServerOperation is not supported for RightClickBuilds

            if hSrc.RightClickBuild
                DAStudio.error('RTW:autosar:serverOperationNotSupported');
            end
            
        end
        
        function verifyShortNames(hSrc)
        % Server Mode

            hModel = hSrc.ModelHandle;
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            maxShortNameLength = get_param(cs, 'AutosarMaxShortNameLength');
        
        
            portName = hSrc.ServerPortName;
                interfaceName = hSrc.ServerInterfaceName;

                if isempty( hSrc.ServerOperationPrototype )
                    DAStudio.error('RTW:autosar:serverOperationPrototypeIsEmpty');
                end

                [fcn, success, err_id, err_msg] = arblk.parseOperationPrototype( hSrc.ServerOperationPrototype, maxShortNameLength );
                if ~success
                    ME = MException( err_id, err_msg);
                    ME.throw;
                end
                testStrs = [...
                            cellstr( portName );...
                            cellstr( interfaceName );...
                            cellstr( fcn.name );...
                           ];

                % prototype name and argument name should not be C keywords
                if ~autosar.validation.AutosarUtils.isNotCKeyword( fcn.name )
                    DAStudio.error('RTW:autosar:notValidOperationPrototypeName', fcn.name);
                end

                for ii=1:numel( fcn.args )
                    if ~autosar.validation.AutosarUtils.isNotCKeyword( fcn.args(ii).name )
                        DAStudio.error('RTW:autosar:notValidOperationPrototypeArgumentName', fcn.args(ii).name);
                    end
                end
           
                

                % Check RuleSNId1, RuleSNId2, RuleSNId3, RuleSNId4, RuleSNId5, RuleSNId6, RuleSNId7
                idcheckmessage = autosar.validation.AutosarUtils.isValidIdentifier(testStrs, 'shortName', maxShortNameLength);
                if ~isempty(idcheckmessage)
                    msg = idcheckmessage;
                    DAStudio.error('RTW:fcnClass:finish', msg);
                end
                
        end

        
        function verifyServerOperationPortCheck(hSrc)
            
            if hSrc.getIsServerOperation
                configData = hSrc.syncWithModel(); 
                portInfo = autosar.validation.CompiledModelUtils.getPortInfo(hSrc);
                
                % ServerOperation
                modelName = get_param(hSrc.ModelHandle, 'Name');
                serverOp = arblk.operationPrototype( modelName, hSrc.ServerOperationPrototype );
                if length( serverOp.argument ) ~= length( configData )
                    DAStudio.error('RTW:autosar:numPortsNotEqualToServerOpArgs');
                end

                for ii = 1:length(configData)
                    % Port checks
                    datatype  = portInfo{ii,6};
                    width     = portInfo{ii,2};
                    direction = portInfo{ii,3};
                    name      = portInfo{ii,5};

                    if direction == 1
                        direction = 'IN';
                        directionStr = 'inport';
                    else
                        direction = 'OUT';
                        directionStr = 'outport';
                    end

                    if width == 1
                        widthStr = '';
                    else
                        widthStr = sprintf('[%d]', width);
                    end

                    portArg = arblk.argumentPrototype( modelName, name, direction, datatype, width );

                    % search for portArg in serverOp
                    isFound = false;
                    for jj = 1:length( serverOp.argument )
                        if serverOp.argument(jj) == portArg
                            isFound = true;
                            break;
                        end
                    end

                    if ~isFound
                        DAStudio.error('RTW:autosar:portNotFoundInServerOp', ...
                                       hSrc.ServerOperationPrototype, direction, ...
                                       datatype, name, widthStr, directionStr, name);
                    end

                end

            end
            
            
        end
        
        function verifyConfigSet(hSrc)
            
            hModel = hSrc.ModelHandle;
            
            if ~strcmp(get_param(hModel, 'MultiInstanceERTCode'), 'on')
                msg = DAStudio.message('RTW:autosar:IsServerOperationNeedsMultiInstanceERTCode');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            % The following are server operation limitations
            if ~strcmp(get_param(hModel, 'InlineParams'), 'on')
                msg = DAStudio.message('RTW:autosar:IsServerOperationNeedsInlineParams');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end
            if ~strcmp(get_param(hModel, 'MultiInstanceErrorCode'), 'Error')
                msg = DAStudio.message('RTW:autosar:IsServerOperationNeedsMultiInstanceErrorCode');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end
            tunableVars = get_param(hModel, 'TunableVars');
            if ~isempty(tunableVars)
                msg = DAStudio.message('RTW:autosar:IsServerOperationHasTunableVars', tunableVars);
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            % make sure that model doesn't contain any client blocks, this is not supported
            % as client blocks are not set up for multiple instantiation turned off and
            % xml isn't generated for these blocks.
            clientPortBlks = arblk.findAUTOSARClientBlks( getfullname(hModel) );
            if ~isempty(clientPortBlks)
                DAStudio.error('RTW:autosar:IsServerOperationHasClientBlk', autosar.validation.AutosarUtils.removeNewLine(clientPortBlks{1}));
            end
            
            
        end

                
        function verifyNoSimulinkGlobals(hSrc)
            

                % get list of referenced Simulink parameters and check that storage class is not SimulinkGlobal
                vars = autosar.validation.CompiledModelUtils.getReferencedWSVars(hSrc.ModelHandle);
                for i=1:length(vars)
                    obj     = vars(i).obj;
                    objName = vars(i).objName;

                    % Check if parameter is a SimulinkGlobal
                    if isa( obj, 'Simulink.Parameter' ) && strcmp( obj.CoderInfo.StorageClass, 'SimulinkGlobal' )
                        DAStudio.error('RTW:autosar:IsServerOperationHasSimulinkGlobalParam', objName);
                    end
                end

            
        end

    end
    
end
