function [modelH, success] = createComponentAsModel(this, ComponentName, varargin)
%CREATECOMPONENTASMODEL creates an AUTOSAR Atomic-Software-Component as a 
%   Simulink model.
%
%   createComponentAsModel(IMPORTEROBJ, 'COMPONENTNAME') creates the Simulink
%   model corresponding to the AUTOSAR Atomic-Software-Component 'COMPONENT' 
%   described in the XML file imported by the arxml.importer object IMPORTEROBJ.
%
%   Usage:
%      [modelH, success] = createComponentAsModel(importerObj,...
%                                                 componentName, varargin)
%
%   Inputs:
%      importerObj:   arxml.importer object
%      componentName: absolute Short-Name path of the Atomic-Software-Component
%      varargin:      Parameter/Value pairs. Supported parameters are:
%
%         CreateSimulinkObject: true or false (default is true). If it is true,
%         this function creates the Simulink.AliasType and Simulink.NumericType 
%         corresponding to the AUTOSAR data types found in the XML file.
%
%         NameConflictAction: 'overwrite' or 'makenameunique' or 'error' (default
%         is 'overwrite'). Use this parameter to control the action to take
%         when a Simulink model with the same name as the component already exists.
%
%         AutoSave: true or false (default is false). If it is true, this function
%         automatically saves the generated Simulink model.
%
%   Outputs:
%      modelH:  Model handle.
%      success: It is true if this function is successful. Otherwise, 
%               it is false.
%
%   See also arxml.importer.createComponentAsSubsystem arxml.importer.getComponentNames
%      arxml.importer.getFile arxml.importer.setFile 
%      arxml.importer.getDependencies arxml.importer.setDependencies

%   Copyright 2007-2015 The MathWorks, Inc.

  
success = false; %#ok
modelH  = -1;    %#ok
  
% Create an instance of the inputParser class.
argParser = inputParser;   

% Add arguments to the schema
argParser.addRequired('this', @(x) isa(x, class(x)));
argParser.addRequired('ComponentName', @ischar);
argParser.addParameter('CreateSimulinkObject', true, @(x)(islogical(x)||(x==1)||(x==0)));
argParser.addParameter('NameConflictAction', 'overwrite', @(x)any(strcmpi(x,{'overwrite','makenameunique','error'})));
argParser.addParameter('AutoSave', false, @(x)(islogical(x)||(x==1)||(x==0)));
argParser.addParameter('CreateInternalBehavior', false, @(x)(islogical(x)||(x==1)||(x==0)));
argParser.addParameter('InitializationRunnable', '', @(x)(ischar(x)));
argParser.addParameter('DataDictionary', '', @(x)(ischar(x)));
argParser.addParameter('PredefinedVariant', '', @(x)(ischar(x)));
argParser.addParameter('SystemConstValueSets', '', @(x)(iscell(x)));

% Read and validate each argument
argParser.parse(this, ComponentName, varargin{:});

% Check if the DataDictionary exists
autosar.mm.mm2sl.utils.checkAndCreateDD(argParser.Results.DataDictionary)
% Create the component
[modelH, success] = p_create_slcomponent(this, argParser, true);

% LocalWords:  IMPORTEROBJ COMPONENTNAME makenameunique BSP
